from unittest.mock import patch

from django.test import TestCase

from ..raven import capture_exception


class RavenTestCase(TestCase):
    @capture_exception
    def func_success(self, arg1, kwarg2):
        return "Good {} {}".format(arg1, kwarg2)

    @capture_exception
    def func_fail(self):
        raise ValueError("Fail")

    def test_success(self):
        with patch("utils.raven.client.captureException") as mock:
            self.assertEqual(
                self.func_success("arg1", kwarg2="kwarg2"), "Good arg1 kwarg2"
            )
        self.assertFalse(mock.called)

    def test_fail(self):
        with patch("utils.raven.client.captureException") as mock:
            with self.assertRaises(ValueError):
                self.func_fail()
        self.assertTrue(mock.called)



from datetime import datetime
from unittest.mock import MagicMock

from django.core.cache import cache
from django.test import TestCase, override_settings
from utils.storage import S3HeadCacheMixin

s3client = MagicMock()
s3client.head_object = MagicMock(
    return_value={"ContentLength": 123, "LastModified": datetime(2015, 1, 1)}
)

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.memcached.MemcachedCache",
        "LOCATION": "127.0.0.1:11211",
    }
}


class MockS3Storage(S3HeadCacheMixin):
    def __init__(self):
        self.s3client = s3client
        self.bucket = "s3-bucket"


@override_settings(CACHES=CACHES)
class StorageTestCase(TestCase):
    def setUp(self):
        self.storage = MockS3Storage()

    def tearDown(self):
        cache.clear()

    def test_cache_save(self):
        data = self.storage.get_head("my_file.txt")
        cache_data = cache.get(self.storage.get_cache_key("my_file.txt"))
        self.assertEqual(cache_data, data)

    def test_reset_head(self):
        self.storage.reset_head("my_file.txt")
        cache_data = cache.get(self.storage.get_cache_key("my_file.txt"))
        self.assertIsNone(cache_data)

    def test_MemcachedKeyCharacterError(self):
        filename = "dir/to/file/weird file with åäö.txt"
        data = self.storage.get_head(filename)
        cache_data = cache.get(self.storage.get_cache_key(filename))
        self.assertEqual(cache_data, data)



import re

import yaml

FM_BOUNDARY = re.compile(r"^---$", re.MULTILINE)


def split(content):
    try:
        _, fm, body = FM_BOUNDARY.split(content, 2)
    except ValueError:
        raise ValueError("Invalid front matter content")
    try:
        fm_data = yaml.safe_load(fm)
    except yaml.scanner.ScannerError:
        raise ValueError("Invalid yaml")
    return [fm_data, body[1:]]



from django.test import TestCase

from ..frontmatter import split

template = """
---
key: value
foo:
  - bar
  - baz
---
Body content
"""

template_invalid = """
---
- this
is
- invalid
---
"""

template_missing_frontmatter = """
Nothing here
"""


class FrontmatterTestCase(TestCase):
    def test_success(self):
        data, body = split(template)
        self.assertEquals(data, {"key": "value", "foo": ["bar", "baz"]})
        self.assertEquals(body, "Body content\n")

    def test_invalid_yaml(self):
        with self.assertRaisesRegex(ValueError, "Invalid yaml"):
            split(template_invalid)

    def test_missing_frontmatter(self):
        with self.assertRaisesRegex(ValueError, "Invalid front matter content"):
            split(template_missing_frontmatter)



import mimetypes
from hashlib import sha256
from io import StringIO

import boto3
from django.conf import settings
from django.contrib.staticfiles.storage import ManifestFilesMixin, StaticFilesStorage
from django.core.cache import cache
from django.core.files.base import ContentFile
from django.core.files.storage import Storage
from django.utils.deconstruct import deconstructible
from pipeline.storage import PipelineMixin


class PipelineManifestStorage(PipelineMixin, ManifestFilesMixin, StaticFilesStorage):
    pass


class S3HeadCacheMixin:
    def get_cache_key(self, name):
        return "storages_meta__{}".format(sha256(name.encode()).hexdigest())

    def get_head(self, name, use_cache=True):
        if use_cache:
            data = cache.get(self.get_cache_key(name))
            if data:
                return data
        data = self.s3client.head_object(Bucket=self.bucket, Key=name)
        cache.set(self.get_cache_key(name), data)
        return data

    def reset_head(self, name):
        cache.delete(self.get_cache_key(name))


@deconstructible
class S3MediaStorage(Storage, S3HeadCacheMixin):
    def __init__(self):
        self.s3client = boto3.client("s3", region_name=settings.AWS_REGION)
        self.bucket = settings.MEDIA_BUCKET

    def delete(self, name):
        self.s3client.delete_object(Bucket=self.bucket, Key=name)

    def exists(self, name):
        try:
            self.get_head(name, use_cache=False)
            return True
        except Exception:
            return False

    def size(self, name):
        return self.get_head(name)["ContentLength"]

    def url(self, name):
        return "https://s3-{aws_region}.amazonaws.com/{bucket}/{key}".format(
            aws_region=settings.AWS_REGION, bucket=self.bucket, key=name
        )

    def get_modified_time(self, name):
        return self.get_head(name)["LastModified"]

    def _open(self, name, mode="rb"):
        response = self.s3client.get_object(Bucket=self.bucket, Key=name)
        return ContentFile(response["Body"].read())

    def _save(self, name, content):
        if isinstance(content.file, StringIO):
            content = ContentFile(content.read().encode("utf-8"), content.name)
        self.s3client.put_object(
            Bucket=self.bucket,
            Key=name,
            Body=content,
            ACL="public-read",
            ContentType=mimetypes.guess_type(name)[0] or "binary/octet-stream",
        )
        self.reset_head(name)
        return name



import logging


class Logger:
    def __init__(self, **kwargs):
        self.options = kwargs
        self.logger = logging.getLogger(kwargs.get("name"))

    def copy(self, **kwargs):
        options = self.options.copy()
        options.update(kwargs)
        return Logger(**options)

    def construct_msg(self, msg):
        return ":".join(
            [
                "{}={}".format(key, value)
                for key, value in self.options.get("fields", {}).items()
            ]
            + [str(msg)]
        )

    def name(self, name):
        return self.copy(name=name)

    def fields(self, **kwargs):
        return self.copy(fields=kwargs)

    def log(self, level, msg, *args, **kwargs):
        self.logger.log(level, self.construct_msg(msg), *args, **kwargs)

    def debug(self, msg, *args, **kwargs):
        self.logger.debug(self.construct_msg(msg), *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        self.logger.info(self.construct_msg(msg), *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        self.logger.warning(self.construct_msg(msg), *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        self.logger.error(self.construct_msg(msg), *args, **kwargs)

    def critical(self, msg, *args, **kwargs):
        self.logger.critical(self.construct_msg(msg), *args, **kwargs)

    def exception(self, msg, *args, **kwargs):
        self.logger.exception(self.construct_msg(msg), *args, **kwargs)


logger = Logger()


def getLogger(name):
    return Logger(name=name)



from django.template import Context, Template
from django.test import TestCase

template = "{% load jsonify %}{{ content | jsonify:2 }}"

template_html = """[
  "foo",
  "bar",
  "ba<"
]"""


class MarkdownTestCase(TestCase):
    def test_markdown(self):
        data = {"content": ["foo", "bar", "ba<"]}
        html = Template(template).render(Context(data))
        self.assertEqual(html, template_html)



from django.conf import settings
from django.urls import reverse


def absolute_reverse(*args, site_url=None, **kwargs):
    if not site_url:
        site_url = settings.SITE_URL
    return "".join([site_url, reverse(*args, **kwargs)])



import json

from django import template
from django.utils.safestring import mark_safe

register = template.Library()


@register.filter(is_safe=True)
def jsonify(obj, indent=0):
    return mark_safe(json.dumps(obj, indent=indent))



from unittest.mock import MagicMock

from django.test import TestCase

from ..middlewares.latency import LatencyMiddleware


def get_response(request):
    return None


class Request:
    def build_absolute_uri(self):
        return "http://example.com"


class LatencyMiddlewareTestCase(TestCase):
    def setUp(self):
        self.middleware = LatencyMiddleware(get_response)

    def test_success(self):
        self.middleware.logger.info = MagicMock()
        self.middleware(Request())
        self.middleware.logger.info.assert_called_with(
            "url: http://example.com Latency 0ms"
        )



from .signing import sign, unsign


class SignedPkMixin(object):
    def get_pk(self):
        try:
            return self.kwargs["pk"]
        except KeyError:
            return unsign(self.kwargs["pk_signed"])

    def get_pk_signed(self):
        try:
            return self.kwargs["pk_signed"]
        except KeyError:
            return sign(self.kwargs["pk"])



from django.template import Context, Template
from django.test import TestCase
from utils.urls import absolute_reverse

template = '{% load absolute_url %}{% absolute_url "test" 1 %}'


class AbsoluteUrlTestCase(TestCase):
    def test_templatetags(self):
        html = Template(template).render(Context({}))
        self.assertEqual(html, "http://www.example.com/test/1/")

    def test_function(self):
        url = absolute_reverse("test", args=[1])
        self.assertEqual(url, "http://www.example.com/test/1/")



from django.core.exceptions import ValidationError
from django.test import TestCase

from ..validators import EmailListValidator


class EmailListValidatorTestCase(TestCase):
    def setUp(self):
        self.validator = EmailListValidator()

    def test_success(self):
        self.validator("foo@example.com bar@example.com")

    def test_fail(self):
        with self.assertRaisesRegex(ValidationError, '"bad" is not a valid email'):
            self.validator("bad foo@example.com")



import functools

from raven.contrib.django.raven_compat.models import client


def capture_exception(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except Exception:
            client.captureException()
            raise

    return wrapper



import json

from django.contrib.postgres.forms.jsonb import InvalidJSONInput, JSONField
from django.forms import ValidationError


class JSONPrettyField(JSONField):
    def __init__(self, *args, **kwargs):
        self.__indent = kwargs.pop("indent", 2)
        self.__dict_only = kwargs.pop("dict_only", False)
        self.__list_only = kwargs.pop("list_only", False)
        if self.__dict_only and self.__list_only:
            raise ValueError("Only one of dict_only or list_only can be True")
        super().__init__(*args, **kwargs)

    def prepare_value(self, value):
        if isinstance(value, InvalidJSONInput):
            return value
        return json.dumps(
            value, indent=self.__indent, sort_keys=True, ensure_ascii=False
        )

    def validate(self, value):
        if self.__dict_only and not isinstance(value, dict):
            raise ValidationError("Value is not of type dict")
        if self.__list_only and not isinstance(value, list):
            raise ValidationError("Value is not of type list")
        return value


class YAMLPrettyField(JSONPrettyField):
    def __init__(self, *args, **kwargs):
        self.__indent = kwargs.pop("indent", 2)
        self.__default_flow_style = kwargs.pop("default_flow_style", False)
        super().__init__(*args, **kwargs)

    def to_python(self, value):
        import yaml

        try:
            return yaml.safe_load(value)
        except yaml.YAMLError as e:
            raise ValidationError(e)

    def prepare_value(self, value):
        import yaml

        try:
            return yaml.safe_dump(
                value,
                indent=self.__indent,
                default_flow_style=self.__default_flow_style,
                allow_unicode=True,
            )
        except Exception:
            return value



import logging
import time


class LatencyMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response
        self.logger = logging.getLogger(__name__)

    def __call__(self, request):
        start_time = time.time()
        try:
            return self.get_response(request)
        finally:
            ds = int((time.time() - start_time) * 1000)
            self.logger.info(
                "url: {} Latency {}ms".format(request.build_absolute_uri(), ds)
            )



from django.template import Context, Template
from django.test import TestCase

template_filter = "{% load md %}{{ content | markdown }}"
template_tag = "{% load md %}{% markdown %}{{ content }}{% endmarkdown %}"
template_tag_as = (
    "{% load md %}{% markdown as my_variable %}{{ content }}{% endmarkdown %}"
)

content = """
# My template

Some test
"""

content_html = """
<h1>My template</h1>
<p>Some test</p>
""".strip()


class MarkdownTestCase(TestCase):
    def test_filter(self):
        data = {"content": content}
        html = Template(template_filter).render(Context(data))
        self.assertEqual(html, content_html)

    def test_template_tag(self):
        data = {"content": content}
        html = Template(template_tag).render(Context(data))
        self.assertEqual(html, content_html)

    def test_template_tag_as(self):
        data = {"content": content}
        context = Context(data)
        html = Template(template_tag_as).render(context)
        self.assertEqual(html, "")
        self.assertIn("my_variable", context)
        self.assertEqual(context["my_variable"], content_html)



import os
from io import StringIO

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import CommandError
from django.test import TestCase


class PipelineTestCase(TestCase):
    def setUp(self):
        self.file_path = os.path.join(settings.STATIC_ROOT, "staticfiles.json")
        if os.path.isfile(self.file_path):
            os.remove(self.file_path)

    def test_success(self):
        call_command("collectstatic", "--noinput", stdout=StringIO())
        call_command("clean_staticfilesjson", stdout=StringIO())
        with open(self.file_path) as f:
            contents = f.read()
            start_content = '{\n    "paths": {\n'
            self.assertTrue(
                contents.startswith(start_content),
                'staticfiles.json doesn\'t start with "{}"'.format(
                    contents[: len(start_content)]
                ),
            )

    def test_missing_staticfilesjson(self):
        with self.assertRaises(CommandError):
            call_command("clean_staticfilesjson", stdout=StringIO())



import json
import os

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    def handle(self, *args, **options):
        file_path = os.path.join(settings.STATIC_ROOT, "staticfiles.json")
        if not os.path.isfile(file_path):
            raise CommandError("Could not find staticfiles.json")
        with open(file_path) as f:
            data = json.load(f)
        with open(file_path, "w") as f:
            json.dump(data, f, indent=4, sort_keys=True)



from unittest.mock import patch

from django.test import TestCase

from ..mixins import SignedPkMixin
from ..signing import UrlSafeSigner, sign, unsign


class ExampleClass(SignedPkMixin):
    def __init__(self, **kwargs):
        self.kwargs = kwargs


class SignedPkMixinTestCase(TestCase):
    def test_pk(self):
        obj = ExampleClass(pk="123")
        with patch("utils.mixins.unsign") as mock:
            self.assertEqual(obj.get_pk(), "123")
        self.assertFalse(mock.called)
        self.assertEqual(unsign(obj.get_pk_signed()), "123")

    def test_pk_signed(self):
        obj = ExampleClass(pk_signed=sign("123"))
        self.assertEqual(obj.get_pk(), "123")
        with patch("utils.mixins.unsign") as mock:
            self.assertEqual(unsign(obj.get_pk_signed()), "123")
        self.assertFalse(mock.called)

    def test_bad_base64(self):
        signer = UrlSafeSigner()
        pk_signed = str(signer.sign(123))
        self.assertIn("=", pk_signed)
        pk_signed = pk_signed.replace("=", "")
        self.assertEqual(signer.unsign(pk_signed), "123")



import yaml
from django import forms
from django.test import TestCase

from ..fields import YAMLPrettyField


class TestForm(forms.Form):
    data_field = YAMLPrettyField(indent=4)


class TestDictForm(forms.Form):
    data_field = YAMLPrettyField(indent=4, dict_only=True)


class TestListForm(forms.Form):
    data_field = YAMLPrettyField(indent=4, list_only=True)


class YAMLPrettyFieldTestCase(TestCase):
    data_dict = "id: 1\ntest: 2"
    data_list = "- id: 1\n- id: 2"

    def test_indentation(self):
        data = {"id": 1, "name": {"first": "Foo", "last": "Bar"}}
        form_data = {"data_field": data}
        form = TestForm(initial=form_data)
        self.assertIn(
            yaml.safe_dump(data, indent=4, default_flow_style=False), form.as_p()
        )

    def test_dict_only_ok(self):
        form_data = {"data_field": self.data_dict}
        form = TestDictForm(form_data)
        self.assertTrue(form.is_valid())

    def test_dict_only_fail(self):
        form_data = {"data_field": self.data_list}
        form = TestDictForm(form_data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors["data_field"], ["Value is not of type dict"])

    def test_list_only_ok(self):
        form_data = {"data_field": self.data_list}
        form = TestListForm(form_data)
        self.assertTrue(form.is_valid())

    def test_list_only_fail(self):
        form_data = {"data_field": self.data_dict}
        form = TestListForm(form_data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors["data_field"], ["Value is not of type list"])

    def test_prepare_value_fail(self):
        form_data = {"data_field": "a: b, c: d"}
        form = TestForm(form_data)
        self.assertIn("a: b, c: d", form.as_p())



import os
from base64 import b64encode
from urllib.parse import quote


def g_b64(data):
    return b"".join([b"data:image/svg+xml;base64,", b64encode(data.encode("utf-8"))])


def g_uri(data):
    return "".join(["data:image/svg+xml;charset=UTF-8,", quote(data)])


def svg_to_data(data):
    d_b64 = g_b64(data)
    d_uri = g_uri(data)
    if len(d_b64) > len(d_uri):
        return d_uri
    return d_b64


def svg_to_data_uri(file_path, include_paths):
    for path in include_paths:
        try:
            with open(os.path.join(path, file_path)) as f:
                return svg_to_data(f.read())
        except FileNotFoundError:
            pass
    raise FileNotFoundError(file_path)



import logging

import requests

from django.core.cache import cache


def shorten(url):
    KEY = "shorturl:{}".format(url)
    logger = logging.getLogger(__name__)
    logger.info("Shortening url {}".format(url))
    shorturl = cache.get(KEY)
    if shorturl:
        logger.info("Returning cached shorturl {}".format(shorturl))
        return shorturl
    data = requests.get(
        "https://is.gd/create.php", params={"format": "json", "url": url}
    ).json()
    if "shorturl" in data:
        shorturl = data["shorturl"]
        logger.info("Shorturl: {}".format(shorturl))
        # Store shorturl for 60 days
        cache.set(KEY, shorturl, 60 * 24 * 3600)
    else:
        logger.warning(data.get("errormessage", "shorturl errormessage was not found"))
        # Store original url for 1 hour
        cache.set(KEY, url, 3600)
        return url
    return shorturl



import re

from django import template
from django.template.defaultfilters import stringfilter
from django.utils.safestring import mark_safe
from markdown import markdown

register = template.Library()


@register.filter("markdown")
@stringfilter
def markdown_filter(value):
    return mark_safe(markdown(value))


@register.tag(name="markdown")
def do_markdown(parser, token):
    nodelist = parser.parse(("endmarkdown",))
    parser.delete_first_token()
    m = re.search(r"as (?P<var_name>\w+)$", token.contents)
    var_name = None
    if m:
        var_name = m.group("var_name")
    return MarkdownNode(nodelist, var_name)


class MarkdownNode(template.Node):
    def __init__(self, nodelist, var_name=None):
        self.nodelist = nodelist
        self.var_name = var_name

    def render(self, context):
        value = markdown_filter(self.nodelist.render(context))
        if self.var_name:
            context[self.var_name] = value
            return ""
        return value



# !/usr/bin/env python

from itertools import chain
from os import path

from setuptools import find_packages, setup

extras_require = {
    "markdown": ["Markdown>=2.6,<3"],
    "pipeline": [
        "django-pipeline>=1.6,<1.7",
        "libsass>=0.11,<0.13",
        "csscompressor>=0.9,<1.0",
    ],
    "raven": ["raven>=6.0,<6.1"],
    "s3": ["boto3>=1.4,<1.5"],
    "frontmatter": ["PyYAML>=3,<4"],
    "json": ["psycopg2-binary>=2.7"],
}

extras_require["all"] = list(chain(extras_require.values()))

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="bulv1ne-django-utils",
    packages=find_packages(exclude=["tests"]),
    version="0.2.7",
    description="Django utils for all kinds of situations",
    long_description=long_description,
    author="Niels Lemmens",
    license="MIT",
    author_email="draso.odin@gmail.com",
    url="https://github.com/bulv1ne/django-utils",
    keywords=["django", "utils"],
    install_requires=["django>=1.10"],
    extras_require=extras_require,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development",
        "Topic :: Utilities",
    ],
)



from django.template import Context, Template
from django.test import TestCase

template = """
{% load md %}
{% markdown %}
# Foo
## Bar
Baz
{% endmarkdown %}
"""

template_html = """
<h1>Foo</h1>
<h2>Bar</h2>
<p>Baz</p>
"""


class MarkdownTestCase(TestCase):
    def test_markdown(self):
        html = Template(template).render(Context({}))
        self.assertHTMLEqual(html, template_html)
